from requests import get
from bs4 import BeautifulSoup
from chardet import detect
import csv
from json import dumps, load
from flask import Flask, request, jsonify, render_template
from datetime import datetime
import threading
import logging

ROOT = "http://192.168.2.20"
app = Flask(__name__)

# Turns a csv reader into a list of dictionaries(json)
def reader_to_json(reader:csv.DictReader) -> list[dict]:
  arr = []
  for row in reader:
    # No whitespaces
    row = {k.strip():v.strip() for k,v in row.items()}
    arr.append(row)
  return arr

# Gets a csv file from the server and turns it into a list of dictionaries(json)
def get_json(url:str) -> list[dict]:
  # get the file
  raw = get(url).content
  print(raw)

  # Figure out the encoding
  encoding = detect(raw)['encoding']
  raw = raw.decode('utf-8')
  
  # turn it into json
  arr = reader_to_json(csv.DictReader(raw.splitlines()))
  
  return arr

# Gets file from the server
def get_file(entries:int) -> dict:
  url = f"{ROOT}/Portal/Portal.mwsl?PriNav=DataLog&RecentCount={entries}"
  soup = BeautifulSoup(get(url).content, "html.parser")
  link = soup.find(id="link1")
  
  if link:
    link = link.get("href")
    link = f"{ROOT}{link}"
    return get_json(link)
  
  raise ValueError("No link found")

  # DEBUG: Local json
  # with open("out.json") as i:
  #   return load(i)
  
# Saves a json file
def save_file(json:list[dict], filename:str) -> None:
  try:
    with open(filename, "w") as o:
      o.write(dumps(json, indent=2))
  except Exception as exc:
    raise IndexError("Error saving file") from exc
  
@app.route("/", methods=["GET"])
def home():
  return render_template("index.html", entries=get_entries(10).json, date=datetime.now())

@app.route("/get", methods=["GET"])
def get_entries(entries:int=None):
  if not entries:
    entries = request.args.get("entries")
    try:
      entries = int(entries)
    except:
      return jsonify({"error":"Invalid entries"})
  try:
    file = get_file(entries)
    return jsonify(file)
  except Exception as exc:
    return jsonify({"error":str(exc)})

@app.route("/dump", methods=["GET"])
def dump():
  with open("out.json") as i:
    return i.read()

@app.route("/dumpLast", methods=["GET"])
def dump_last():
  with open("out.json") as i:
    # Return last 50 entries
    return dumps(load(i)[-50:])
  
  
def pull_data():
  logging.info("Pulling data")
  threading.Timer(86400, pull_data).start()
  save_file(get_file(50), "out.json")
  


if __name__ == "__main__":
  # Add threading
  pull_data()
  app.run(debug=True)